@@ -12,13 +12,13 @@ from pydantic import BaseModel, Field, ValidationError
 
 
 class ExtractConfig(BaseModel):
-    extract_active: bool = True
+    active: bool = True
 
 class TransformConfig(BaseModel):
-    transform_active: bool = True
+    active: bool = True
     
 class LoadConfig(BaseModel):
-    load_active: bool = True
+    active: bool = True
     
 class ConfigBase(BaseModel):
     """Generic, permissive pipeline model for extract-only use cases."""
