@@ -219,8 +219,15 @@ class ETLDuckDBHandler:
     def query(self, sql: str, params: Optional[Iterable[Any]] = None):
         return self.con.execute(sql, params or [])
 
-    def latest_table_name(self, steps: type[Enum], entity: str) -> str | None:
-        for step in reversed(list(steps)):
+    def latest_table_name(
+        self, steps: type[Enum], maxstep: Enum | None, entity: str
+    ) -> str | None:
+
+        step_list = list(steps)
+        max_index = (step_list.index(maxstep) if maxstep else len(step_list)) - 1
+
+        # iterate backwards up to that index (inclusive)
+        for step in reversed(step_list[: max_index + 1]):
             table_name = f"{step.value}_{entity}"
             if self.table_exists(table_name):
                 return table_name
@@ -244,7 +251,6 @@ class ETLDuckDBHandler:
                 COPY "{table_name}" TO '{csv_path.as_posix()}'
                 (HEADER, DELIMITER ';', QUOTE '"', ESCAPE '\\', FORCE_QUOTE ({force_quote_cols}));
             """
-            self.logger.info(f"Exporting table {table_name} to CSV at {csv_path} using query:\n{query}")
             self.con.execute(query)
             self.logger.info(
                 f"Uploading table {table_name} to Nemo project {project_name} from {csv_path}"
@@ -407,3 +413,169 @@ class ETLDuckDBHandler:
 
         self.logger.info(f"Ingested into {tname}. Row count: {rowcount:,}")
         return rowcount
+
+    def ingest_csv(
+        self,
+        filename: str | Path,
+        table_name: str | None = None,
+        create_mode: str = "replace",     # 'replace' or 'append'
+
+        # Core schema/structure
+        header: bool = True,
+        columns: list[str] | None = None,  # required when header=False
+        # Delimiters & quoting
+        separator: str | None = None,      # maps to DELIM
+        quote: str | None = None,          # maps to QUOTE
+        escape: str | None = None,         # maps to ESCAPE
+        comment: str | None = None,        # comment line prefix (e.g. '#')
+        newline: str | None = None,        # 'auto', 'lf', 'crlf', 'cr'
+        # Formats
+        dateformat: str | None = None,     # e.g. 'YYYY-MM-DD'
+        timestampformat: str | None = None,# e.g. 'YYYY-MM-DD HH:MI:SS'
+        decimal_separator: str | None = None, # ',' or '.'
+        thousands: str | None = None,         # '.' or ','
+        # Encoding / compression
+        encoding: str | None = None,       # e.g. 'UTF8', 'ISO-8859-1'
+        compression: str | None = None,    # 'gzip', 'zstd', 'auto', ...
+        # Detection & typing
+        auto_detect: bool | None = None,   # let DuckDB infer schema (default: True in read_csv_auto)
+        all_varchar: bool | None = None,   # force all columns to VARCHAR
+        normalize_names: bool | None = None, # convert names to valid identifiers
+        # Performance & robustness
+        sample_size: int | None = None,    # rows to sample for type inference
+        max_line_size: int | None = None,  # max bytes per line
+        ignore_errors: bool | None = None, # skip malformed rows
+        parallel: bool | None = None,      # enable/disable parallel CSV reader
+        hive_partitioning: bool | None = None, # infer partitions from path
+        filename_col: str | None = None,   # add a column with source filename
+    ) -> int:
+        """
+        Highly-configurable CSV ingest into DuckDB via read_csv_auto().
+        - Supports most read_csv_auto OPTIONS via Python args
+        - Handles header/columns contract:
+            * header=True  -> first row are column names; 'columns' must be None
+            * header=False -> 'columns' must be provided as names
+        - Gzip (.csv.gz) and other compressions supported (auto or explicit)
+        """
+        
+        def _sql_str_literal(s: str | None) -> str:
+            """Escape a Python string as a single-quoted SQL literal (or return NULL)."""
+            if s is None:
+                return "NULL"
+            return "'" + s.replace("'", "''") + "'"
+
+        def _sql_bool_literal(b: bool | None) -> str:
+            """Return SQL TRUE/FALSE/NULL literal for a Python bool."""
+            if b is None:
+                return "NULL"
+            return "TRUE" if b else "FALSE"
+
+        def _sql_list_literal(items: Iterable[str] | None) -> str:
+            """Return a DuckDB list literal like ['a','b'] or NULL for None/empty."""
+            if not items:
+                return "NULL"
+            escaped = ", ".join(_sql_str_literal(x) for x in items)
+            return f"[{escaped}]"
+        
+        # Resolve & validate path
+        src = Path(filename)
+        if not src.exists():
+            raise FileNotFoundError(f"CSV file not found: {src}")
+
+        # Validate header/columns contract
+        if header and columns:
+            raise ValueError("When header=True, 'columns' must be None.")
+        if not header and not columns:
+            raise ValueError("When header=False, you must provide 'columns' (list of names).")
+
+        # Pick table name (default: filename stem)
+        tname = _safe_table_name(table_name or src.stem)
+        self.logger.info(f'Ingesting CSV into DuckDB: file={src} -> table="{tname}"')
+
+        # Build read_csv_auto options dynamically (only include those explicitly set)
+        opts: list[str] = []
+
+        # Header + names
+        opts.append(f"HEADER={_sql_bool_literal(header)}")
+        if not header and columns is not None:
+            opts.append(f"NAMES={_sql_list_literal(columns)}")
+
+        # Delimiters & quoting
+        if separator is not None:
+            opts.append(f"DELIM={_sql_str_literal(separator)}")
+        if quote is not None:
+            opts.append(f"QUOTE={_sql_str_literal(quote)}")
+        if escape is not None:
+            opts.append(f"ESCAPE={_sql_str_literal(escape)}")
+        if comment is not None:
+            opts.append(f"COMMENT={_sql_str_literal(comment)}")
+        if newline is not None:
+            opts.append(f"NEWLINE={_sql_str_literal(newline)}")  # 'auto'|'lf'|'crlf'|'cr'
+
+        # Formats
+        if dateformat is not None:
+            opts.append(f"DATEFORMAT={_sql_str_literal(dateformat)}")
+        if timestampformat is not None:
+            opts.append(f"TIMESTAMPFORMAT={_sql_str_literal(timestampformat)}")
+        if decimal_separator is not None:
+            opts.append(f"DECIMAL_SEPARATOR={_sql_str_literal(decimal_separator)}")
+        if thousands is not None:
+            opts.append(f"THOUSANDS={_sql_str_literal(thousands)}")
+
+        # Encoding / compression
+        if encoding is not None:
+            opts.append(f"ENCODING={_sql_str_literal(encoding)}")
+        if compression is not None:
+            opts.append(f"COMPRESSION={_sql_str_literal(compression)}")
+
+        # Detection & typing
+        if auto_detect is not None:
+            opts.append(f"AUTO_DETECT={_sql_bool_literal(auto_detect)}")
+        if all_varchar is not None:
+            opts.append(f"ALL_VARCHAR={_sql_bool_literal(all_varchar)}")
+        if normalize_names is not None:
+            opts.append(f"NORMALIZE_NAMES={_sql_bool_literal(normalize_names)}")
+
+        # Performance & robustness
+        if sample_size is not None:
+            if sample_size < 0:
+                raise ValueError("sample_size must be >= 0")
+            opts.append(f"SAMPLE_SIZE={sample_size}")
+        if max_line_size is not None:
+            if max_line_size <= 0:
+                raise ValueError("max_line_size must be > 0")
+            opts.append(f"MAX_LINE_SIZE={max_line_size}")
+        if ignore_errors is not None:
+            opts.append(f"IGNORE_ERRORS={_sql_bool_literal(ignore_errors)}")
+        if parallel is not None:
+            opts.append(f"PARALLEL={_sql_bool_literal(parallel)}")
+        if hive_partitioning is not None:
+            opts.append(f"HIVE_PARTITIONING={_sql_bool_literal(hive_partitioning)}")
+        if filename_col is not None:
+            # DuckDB option is FILENAME=1 plus a column name in either 'filename' or 'filename=colname' depending on version.
+            # The modern way is FILENAME=colname (string), older is FILENAME=1 + adding column later.
+            opts.append(f"FILENAME={_sql_str_literal(filename_col)}")
+
+        # Compose the read_csv_auto() call
+        options_sql = ", ".join(opts)
+        base_sql = f"SELECT * FROM read_csv_auto(?, {options_sql});"
+        params: list[Any] = [str(src)]
+
+        # Execute DDL/DML
+        if create_mode == "replace":
+            self.con.execute(f'DROP TABLE IF EXISTS "{tname}";')
+            self.con.execute(f'CREATE TABLE "{tname}" AS {base_sql}', params)
+        elif create_mode == "append":
+            if not self.table_exists(tname):
+                # Create empty table first to capture inferred schema
+                self.con.execute(
+                    f'CREATE TABLE "{tname}" AS SELECT * FROM ({base_sql[:-1]}) AS s LIMIT 0;',
+                    params,
+                )
+            self.con.execute(f'INSERT INTO "{tname}" {base_sql}', params)
+        else:
+            raise ValueError("create_mode must be 'replace' or 'append'")
+
+        rowcount = self.count_rows(tname)
+        self.logger.info(f'Ingested into "{tname}". Row count: {rowcount:,}')
+        return rowcount
\ No newline at end of file
