@@ -9,8 +9,5 @@ class ETLStep(Enum):
 class ETLAdapter(Enum):
     GEDYS = "gedys"
     ZENTIS = "zentis"
-    GENERICODBC = "genericodbc"
-    INFORCOM = "inforcom"
-    SAPECC = "sapecc"
 
 
