@@ -3,17 +3,17 @@ from pydantic import BaseModel, Field
 
 
 class ExtractConfigHubSpotForecast(BaseModel):
-    extract_active: bool = True
+    active: bool = True
     deal_pipelines: Optional[List[str]] = Field(
         default_factory=lambda: ["*"],
         description="List of deal pipelines to include. Use ['*'] to include all.",
     )
 
 class TransformConfigHubSpotForecast(BaseModel):
-    transform_active: bool = True
+    active: bool = True
 
 class LoadConfigHubSpotForecast(BaseModel):
-    load_active: bool = True
+    active: bool = True
     forecast_call_xlsx_file: str = "FCCall.xlsx"
     
 class ConfigHubSpotForecast(BaseModel):
