@@ -49,7 +49,7 @@ def hubspotforecast_flow(args) -> None:
         - Extract settings: table specifications and activation flags
         - Transform settings: business rules and data processing parameters
         - Load settings: target system configuration and batch parameters
-        - Global settings: phase activation flags (extract_active, transform_active, load_active)
+        - Global settings: phase activation flags (active, active, active)
 
     Error Handling:
         Any exceptions raised during the ETL process will be logged and propagated by Prefect,
@@ -87,15 +87,15 @@ def hubspotforecast_flow(args) -> None:
     fh: ETLFileHandler = ETLFileHandler(nl=nl, cfg=cfg, logger=logger)
 
     # run steps
-    if cfg.extract.extract_active:
+    if cfg.extract.active:
         logger.info("Extracting objects from HubSpotForecast")
         extract(nl=nl, cfg=cfg, logger=logger, fh=fh)
 
-    if cfg.transform.transform_active:
+    if cfg.transform.active:
         logger.info("Transforming HubSpotForecast objects")
         transform(nl=nl, cfg=cfg, logger=logger, fh=fh)
 
-    if cfg.load.load_active:
+    if cfg.load.active:
         logger.info("Loading HubSpotForecast objects")
         load(nl=nl, cfg=cfg, logger=logger, fh=fh)
 
