@@ -90,39 +90,6 @@ class MigManTransform:
 
         super().__init__()
 
-    def transform(self) -> None:
-        """
-        Execute the main transformation process for MigMan data.
-
-        This method orchestrates the complete transformation process by:
-        1. Loading extracted data from the previous ETL phase
-        2. Applying data validation and quality checks
-        3. Performing data type conversions and formatting
-        4. Applying business rules and logic
-        5. Creating calculated fields and data enrichment
-        6. Ensuring data consistency and integrity
-        7. Preparing data for the loading phase
-
-        The method provides detailed logging for monitoring and debugging purposes
-        and handles errors gracefully to ensure pipeline stability.
-
-        Note:
-            The actual transformation logic needs to be implemented based on
-            the specific MigMan system requirements and business rules.
-        """
-        self.logger.info("Transforming all MigMan objects")
-
-        # transform objects
-
-        # start with joins. After this step, we have all data in our data modell
-        self.joins()
-
-        # then remove empty columns
-        self.nonempty()
-        
-        # then handle duplicates
-        self.duplicates()
-
     def joins(self) -> None:
         """
         Execute join operations for MigMan data transformation.
@@ -148,38 +115,34 @@ class MigManTransform:
             self.logger.info("Join configuration is inactive, skipping joins")
             return
 
-        adapter = self.cfg.transform.join.adapter
-        if not adapter or adapter == "":
-            raise ValueError("No adapter specified for joins")
-        
-        self.logger.info(f"Using adapter: {adapter}")
+        self.logger.info(f"Using adapter: {self.cfg.setup.adapter}")
+
+        for project in self.cfg.setup.projects:
+
+            # get join configuration for the project
+            join_cfg = self.cfg.transform.join.joins.get(project, None)
+            if join_cfg is None:
+                raise ValueError(f"No join configuration found for project: {project}")
 
-        for join_name, join_cfg in self.cfg.transform.join.joins.items():
             if not join_cfg.active:
-                self.logger.info(f"Skipping inactive join: {join_name}")
+                self.logger.info(f"Skipping inactive join: {project}")
                 continue
 
-            self.logger.info(f"Processing join: {join_name}")
+            self.logger.info(f"Processing join: {project}")
 
-            # load the JOIN SQL from the config
-            file = (
-                resources.files("nemo_library_etl")
-                / "adapter"
-                / "migman"
-                / "config"
-                / "joins"
-                / adapter
-                / join_cfg.file
+            query = MigManUtils.getJoinQuery(
+                adapter=self.cfg.setup.adapter, join_file=join_cfg.file
             )
 
-            with resources.as_file(file) as sql_file:
-                query = sql_file.read_text(encoding="utf-8")
-
-            if not query:
-                raise ValueError(f"Join SQL file is empty: {join_cfg.file}")
+            # if we have configured a limit, apply it to the query
+            if self.cfg.transform.join.limit is not None:
+                self.logger.info(
+                    f"Applying limit of {self.cfg.transform.join.limit} to join query"
+                )
+                query += f"\nLIMIT {self.cfg.transform.join.limit}\n"
 
             # add result_creation to the query
-            table_name = MigManTransformStep.JOINS.value + "_" + join_name
+            table_name = MigManTransformStep.JOINS.value + "_" + project
             query = f'CREATE OR REPLACE TABLE "{table_name}" AS\n' + query
 
             # Execute the join query
@@ -190,8 +153,9 @@ class MigManTransform:
                 f"SELECT name FROM pragma_table_info('{table_name}')"
             ).fetchall()
             columns = [col[0] for col in columns]
+            project_name, postfix = MigManUtils.split_migman_project_name(project)
             MigManUtils.validate_columns(
-                project=join_name, postfix="", columns=columns, missing_ok=True
+                project=project_name, postfix=postfix, columns=columns, missing_ok=True
             )
 
             # export results from database
@@ -201,7 +165,7 @@ class MigManTransform:
                     fh=self.fh,
                     step=ETLStep.TRANSFORM,
                     substep=MigManTransformStep.JOINS,
-                    entity=join_name,
+                    entity=project,
                     gzip_enabled=False,
                 )
 
@@ -212,21 +176,208 @@ class MigManTransform:
                     delete_temp_files=self.cfg.transform.delete_temp_files,
                 )
 
+    def mappings(self) -> None:
+        """
+        Apply mappings to the MigMan data.
+
+        This method processes the MigMan data by applying predefined mappings
+        to transform the data according to business rules and requirements.
+        It ensures that the data is correctly formatted and enriched for
+        subsequent loading into the target system.
+
+        Note:
+            The actual mapping logic needs to be implemented based on
+            the specific MigMan system requirements and mapping definitions.
+        """
+        self.logger.info("Applying mappings to MigMan data")
+
+        if not self.cfg.transform.mapping.active:
+            self.logger.info("Mapping configuration is inactive, skipping mappings")
+            return
+
+        for mapping in self.cfg.transform.mapping.mappings:
+
+            if not mapping.active:
+                self.logger.info(f"Mapping {mapping} is inactive, skipping")
+                continue
+
+            self.logger.info(f"Processing mapping: {mapping}")
+
+            # Implement mapping logic here
+        
+
+    def newnumbers(self) -> None:
+        """
+        Assign new numbers to MigMan data records.
+
+        This method generates and assigns new unique identifiers or numbers
+        to the MigMan data records as part of the transformation process.
+        It ensures that each record has a distinct identifier for tracking
+        and reference in the target system.
+
+        Note:
+            The actual new number assignment logic needs to be implemented based on
+            the specific MigMan system requirements and numbering schemes.
+        """
+        self.logger.info("Assigning new numbers to MigMan data")
+
+        if not self.cfg.transform.newnumbers.active:
+            self.logger.info("NewNumbers configuration is inactive, skipping new numbers")
+            return
+
+        # Placeholder for new number assignment logic
+        self.logger.info("New number assignment logic is not yet implemented.")
+        
     def nonempty(self) -> None:
         """
         Remove empty columns from the MigMan data.
+
+        This method identifies and removes columns that are completely empty (NULL or empty strings)
+        from the transformed data tables. It operates directly in DuckDB for memory efficiency.
         """
         self.logger.info("Removing empty columns from MigMan data")
 
-        if self.cfg.transform.nonempty is None:
-            self.logger.info("No nonempty configuration found, skipping nonempty")
-            return
-
         if not self.cfg.transform.nonempty.active:
             self.logger.info("Nonempty configuration is inactive, skipping nonempty")
             return
 
+        for project in self.cfg.setup.projects:
 
+            table = self.local_database.latest_table_name(
+                steps=MigManTransformStep,
+                maxstep=MigManTransformStep.NONEMPTY,
+                entity=project,
+            )
+            if table is None:
+                raise ValueError(f"No table found for entity {project}")
+
+            self.logger.info(
+                f"Processing nonempty for project: {project}, table: {table}"
+            )
+
+            # Get all column names and their data types
+            columns_info = self.local_database.con.execute(
+                f"SELECT name, type FROM pragma_table_info('{table}')"
+            ).fetchall()
+
+            if not columns_info:
+                raise ValueError(f"Failed to retrieve columns for table {table}")
+
+            # Identify empty columns by checking if all values are NULL or empty strings
+            empty_columns = []
+            non_empty_columns = []
+
+            self.logger.info(f"Analyzing {len(columns_info)} columns for emptiness...")
+
+            for column_name, column_type in columns_info:
+                # Build condition to check if column is completely empty
+                # For string types, check both NULL and empty string
+                if column_type.upper() in ["VARCHAR", "TEXT", "CHAR"]:
+                    empty_check_query = f"""
+                        SELECT COUNT(*) 
+                        FROM "{table}" 
+                        WHERE "{column_name}" IS NOT NULL 
+                        AND TRIM("{column_name}") != ''
+                    """
+                else:
+                    # For non-string types, only check for NULL
+                    empty_check_query = f"""
+                        SELECT COUNT(*) 
+                        FROM "{table}" 
+                        WHERE "{column_name}" IS NOT NULL
+                    """
+
+                non_empty_count = self.local_database.con.execute(
+                    empty_check_query
+                ).fetchone()[0]
+
+                if non_empty_count == 0:
+                    empty_columns.append(column_name)
+                    self.logger.debug(f"Column '{column_name}' is empty")
+                else:
+                    non_empty_columns.append(column_name)
+
+            self.logger.info(
+                f"Found {len(empty_columns)} empty columns out of {len(columns_info)} total columns"
+            )
+
+            if empty_columns:
+                self.logger.info(f"Removing empty columns: {', '.join(empty_columns)}")
+
+                # Create new table with only non-empty columns
+                if non_empty_columns:
+                    # Build SELECT statement with non-empty columns
+                    select_columns = ", ".join(
+                        [f'"{col}"' for col in non_empty_columns]
+                    )
+                    new_table_name = f"{MigManTransformStep.NONEMPTY.value}_{project}"
+
+                    create_query = f"""
+                        CREATE OR REPLACE TABLE "{new_table_name}" AS
+                        SELECT {select_columns}
+                        FROM "{table}"
+                    """
+
+                    self.local_database.query(create_query)
+
+                    # Verify the new table
+                    new_row_count = self.local_database.con.execute(
+                        f'SELECT COUNT(*) FROM "{new_table_name}"'
+                    ).fetchone()[0]
+
+                    self.logger.info(
+                        f"Created table '{new_table_name}' with {len(non_empty_columns)} columns and {new_row_count:,} rows"
+                    )
+
+                    # Export results if configured
+                    if self.cfg.transform.dump_files:
+                        self.local_database.export_table(
+                            table_name=new_table_name,
+                            fh=self.fh,
+                            step=ETLStep.TRANSFORM,
+                            substep=MigManTransformStep.NONEMPTY,
+                            entity=project,
+                            gzip_enabled=False,
+                        )
+
+                    # Upload to Nemo if configured
+                    if self.cfg.transform.load_to_nemo:
+                        self.local_database.upload_table_to_nemo(
+                            table_name=new_table_name,
+                            project_name=f"{self.cfg.transform.nemo_project_prefix}{new_table_name}",
+                            delete_temp_files=self.cfg.transform.delete_temp_files,
+                        )
+                else:
+                    self.logger.warning(
+                        f"All columns in table '{table}' are empty - cannot create table with no columns"
+                    )
+            else:
+                self.logger.info(
+                    f"No empty columns found in table '{table}' - creating copy for consistency"
+                )
+
+                # Create a copy of the table for consistency in the pipeline
+                new_table_name = f"{MigManTransformStep.NONEMPTY.value}_{project}"
+                copy_query = f'CREATE OR REPLACE TABLE "{new_table_name}" AS SELECT * FROM "{table}"'
+                self.local_database.query(copy_query)
+
+                # Export and upload the unchanged table if configured
+                if self.cfg.transform.dump_files:
+                    self.local_database.export_table(
+                        table_name=new_table_name,
+                        fh=self.fh,
+                        step=ETLStep.TRANSFORM,
+                        substep=MigManTransformStep.NONEMPTY,
+                        entity=project,
+                        gzip_enabled=False,
+                    )
+
+                if self.cfg.transform.load_to_nemo:
+                    self.local_database.upload_table_to_nemo(
+                        table_name=new_table_name,
+                        project_name=f"{self.cfg.transform.nemo_project_prefix}{new_table_name}",
+                        delete_temp_files=self.cfg.transform.delete_temp_files,
+                    )
 
     def duplicates(self) -> None:
         """
@@ -248,18 +399,11 @@ class MigManTransform:
         """
         self.logger.info("Handling duplicates in MigMan data")
 
-        if self.cfg.transform.duplicate is None:
-            self.logger.info("No duplicate configuration found, skipping duplicates")
-            return
-
         if not self.cfg.transform.duplicate.active:
             self.logger.info("Duplicate configuration is inactive, skipping joins")
             return
 
         # create UDFs in DuckDB
-        if not self.local_database or not self.local_database.con:
-            raise ValueError("Database handler is not initialized")
-
         def normalize_text(s: str) -> str:
             if s is None:
                 return ""
@@ -286,6 +430,13 @@ class MigManTransform:
             if model.active is False:
                 self.logger.info(f"Skipping inactive duplicate model: {duplicate_name}")
                 continue
+
+            if not duplicate_name in self.cfg.setup.projects:
+                self.logger.warning(
+                    f"Duplicate model {duplicate_name} not in configured projects, skipping"
+                )
+                continue
+
             self._perform_duplicate_check(duplicate_name, model)
 
     def _perform_duplicate_check(
@@ -304,7 +455,9 @@ class MigManTransform:
         start_time = datetime.now()
 
         table = self.local_database.latest_table_name(
-            steps=MigManTransformStep, entity=duplicate_name
+            steps=MigManTransformStep,
+            maxstep=MigManTransformStep.DUPLICATES,
+            entity=duplicate_name,
         )
         if table is None:
             raise ValueError(f"No table found for entity {duplicate_name}")
