@@ -2,13 +2,13 @@ from pydantic import BaseModel, Field
 
 
 class ExtractConfigRepIntercompany(BaseModel):
-    extract_active: bool = True
+    active: bool = True
 
 class TransformConfigRepIntercompany(BaseModel):
-    transform_active: bool = True
+    active: bool = True
 
 class LoadConfigRepIntercompany(BaseModel):
-    load_active: bool = True
+    active: bool = True
     analyze_table: bool = True
 
     
