@@ -2,13 +2,13 @@ from pydantic import BaseModel, Field
 
 
 class ExtractConfigRepPL(BaseModel):
-    extract_active: bool = True
+    active: bool = True
 
 class TransformConfigRepPL(BaseModel):
-    transform_active: bool = True
+    active: bool = True
 
 class LoadConfigRepPL(BaseModel):
-    load_active: bool = True
+    active: bool = True
     
 class ConfigRepPL(BaseModel):
 
