@@ -2,13 +2,13 @@ from pydantic import BaseModel, Field
 
 
 class ExtractConfigRepPrices(BaseModel):
-    extract_active: bool = True
+    active: bool = True
 
 class TransformConfigRepPrices(BaseModel):
-    transform_active: bool = True
+    active: bool = True
 
 class LoadConfigRepPrices(BaseModel):
-    load_active: bool = True
+    active: bool = True
     
 class ConfigRepPrices(BaseModel):
 
